/* $Id: et4000_mmio.inc,v 1.4 1998/11/24 18:28:09 marcus Exp $
******************************************************************************
	ET4000 MMIO
******************************************************************************

   Copyright (C) 1995 by Jason McMullan, [jmcc@ggi-project.org]
   Copyright (C) 1997 by Andreas Vogler, [duff@allgaeu.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
	This code was derived from the following source of information:
	[1]	IBM VGA driver by Jason McMullan
	[2]	VGADOCS by Finn Thoegersen
*/

#define MM_PROT_RW     (VM_READ|VM_WRITE|VM_SHARED)
#define MM_PROT_W      (VM_WRITE|VM_SHARED)
#define MMAP_IBM_VGA_4 (MMAP_PRIVATE | 4)

static void select_segment(int segment)
{
	kgi_uint hi;

	if(chip>=ET4000_W32) {
		hi = segment & 0x30;
		io_out8(hi | (hi >> 4), EXT_BANK);
	}
	segment &= 0x0f;
	io_out8(segment | (segment << 4), SEG_SELECT);
}

static void mmap_setpage_graphics(struct kgi_mmio_region *mm, kgi_uint offset)
{
	int page=(offset>>16) & 0x3F;
	
	select_segment(page);	
	
	mm->offset=(page<<16);
}

static struct kgi_mmio_region mm_text16= {
	NULL,		/* Next */
	"text16",	/* Name */
	MMAP_FRAMEBUFFER,/* Type (Linear) */
	MM_PROT_RW,	/* Protection map */
	
	0x10000,	/* Total size of area */
	0x10000,	/* Physical size (64k) */
	0xA0000,	/* Physical address of the windows */
	0,		/* Logical offset */
	
	NULL		/* Set offset function */
};

static struct kgi_mmio_region mm_graphics= {
	NULL,		/* Next */
	"ET4000",		/* Name */
	MMAP_FRAMEBUFFER,/* Type (Linear) */
	MM_PROT_RW,	/* Protection map */
	
	0x0,		/* Total size of area (depends on mode)*/
	0x10000,	/* Physical size (64k) */
	0xA0000,	/* Physical address of the windows */
	0,		/* Logical offset */
	
	mmap_setpage_graphics	/* Set offset function */
};

static struct kgi_mmio_region mm_linear_graphics= {
	NULL,		/* Next */
	"ET4000 W32 linear", /* Name */
	MMAP_FRAMEBUFFER,/* Type (Linear) */
	MM_PROT_RW,	/* Protection map */
	
	0,		/* Total size of area (depends on mode)*/
	0,		/* Physical size (4 M) */
	0x0,		/* Physical address of the windows (autodetected)*/
	0,		/* Logical offset */
	
	NULL		/* Set offset function */
};



